@@ -24,6 +24,7 @@
 #include <zim/entry.h>
 #include <zim/item.h>
 #include <zim/error.h>
+#include <zim/tools.h>
 #include "fileimpl.h"
 #include "tools.h"
 #include "log.h"
@@ -32,17 +33,51 @@ log_define("zim.archive")
 
 namespace zim
 {
+  OpenConfig::OpenConfig()
+    :
+        m_preloadXapianDb(true),
+        m_preloadDirentRanges(DIRENT_LOOKUP_CACHE_SIZE)
+    { }
+
   Archive::Archive(const std::string& fname)
-    : m_impl(new FileImpl(fname))
+    : Archive(fname, OpenConfig())
+    { }
+
+  Archive::Archive(const std::string& fname, OpenConfig openConfig)
+    : m_impl(new FileImpl(fname, openConfig))
     { }
 
 #ifndef _WIN32
   Archive::Archive(int fd)
-    : m_impl(new FileImpl(fd))
+    : Archive(fd, OpenConfig())
+    { }
+
+  Archive::Archive(int fd, OpenConfig openConfig)
+    : m_impl(new FileImpl(fd, openConfig))
+    { }
+
+  Archive::Archive(FdInput fd)
+    : Archive(fd, OpenConfig())
+    { }
+
+  Archive::Archive(FdInput fd, OpenConfig openConfig)
+    : m_impl(new FileImpl(fd, openConfig))
     { }
 
   Archive::Archive(int fd, offset_type offset, size_type size)
-    : m_impl(new FileImpl(fd, offset_t(offset), zsize_t(size)))
+    : Archive(FdInput(fd, offset, size), OpenConfig())
+    { }
+
+  Archive::Archive(int fd, offset_type offset, size_type size, OpenConfig openConfig)
+    : Archive(FdInput(fd, offset, size), openConfig)
+    { }
+
+  Archive::Archive(const std::vector<FdInput>& fds)
+    : Archive(fds, OpenConfig())
+    { }
+
+  Archive::Archive(const std::vector<FdInput>& fds, OpenConfig openConfig)
+    : m_impl(new FileImpl(fds, openConfig))
     { }
 #endif
 
@@ -107,7 +142,7 @@ namespace zim
 
   Item Archive::getMetadataItem(const std::string& name) const
   {
-    auto r = m_impl->findx('M', name);
+    auto r = m_impl->findxMetadata(name);
     if (!r.first) {
       throw EntryNotFound("Cannot find metadata");
     }
@@ -127,7 +162,7 @@ namespace zim
     auto end = m_impl->getNamespaceEndOffset('M');
     for (auto idx=start; idx!=end; idx++) {
       auto dirent = m_impl->getDirent(idx);
-      ret.push_back(dirent->getUrl());
+      ret.push_back(dirent->getPath());
     }
     return ret;
   }
@@ -146,25 +181,19 @@ namespace zim
   }
 
   Item Archive::getIllustrationItem(unsigned int size) const {
-    std::ostringstream ss;
-    ss  << "Illustration_" << size << "x" << size << "@" << 1;
-    auto r = m_impl->findx('M', ss.str());
+    // XXX: A square illustration of the requested size may exist with
+    // XXX: non-empty attributes or at a scale different from 1. Shouldn't
+    // XXX: we test for it too, if the {size}x{size}@1 one is not available?
+    return getIllustrationItem(IllustrationInfo{size, size, 1.0, {}});
+  }
+
+  Item Archive::getIllustrationItem(const IllustrationInfo& ii) const {
+    auto r = m_impl->findx('M', ii.asMetadataItemName());
     if (r.first) {
       return getEntryByPath(entry_index_type(r.second)).getItem();
     }
-    // We haven't found the exact entry. Let's "search" for a illustration and
-    // use the first one we found.
-#if 0
-    // We have decided to not implement fallback in case of wrong resolution for now.
-    // We keep this code for reference.
-    r = m_impl->findx('M', "Illustration");
-    auto entry = getEntryByPath(entry_index_type(r.second));
-    if (entry.getPath().find("Illustration") == 0) {
-      return entry.getItem();
-    }
-#endif
     // For 48x48 illustration, return favicon for older zims.
-    if (size == 48) {
+    if ( ii == IllustrationInfo{48, 48, 1.0, {}} ) {
       auto r = findFavicon(*m_impl);
       return getEntryByPath(entry_index_type(r.second)).getItem(true);
     }
@@ -199,6 +228,45 @@ namespace zim
     return ret;
   }
 
+  Archive::IllustrationInfos Archive::getIllustrationInfos() const {
+    IllustrationInfos r;
+    for(auto e = m_impl->findx('M', "Illustration_").second; ; ++e ) {
+      try {
+        const auto path = getEntryByPath(entry_index_type(e)).getPath();
+        if (path.find("Illustration_") != 0) {
+          break;
+        }
+        try {
+          r.push_back(zim::IllustrationInfo::fromMetadataItemName(path));
+        } catch (...) {}
+      } catch (const std::out_of_range& e) {
+        break;
+      }
+    }
+    const IllustrationInfo faviconLikeIllustration{48, 48, 1.0, {}};
+    if (std::find(r.begin(), r.end(), faviconLikeIllustration) == r.end()) {
+      try {
+        // raise a exception if we cannot find the (old format) favicon.
+        findFavicon(*m_impl);
+        r.push_back(faviconLikeIllustration);
+      } catch(EntryNotFound&) {}
+    }
+    return r;
+  }
+
+  Archive::IllustrationInfos Archive::getIllustrationInfos(uint32_t w,
+                                                           uint32_t h,
+                                                           float minScale) const
+  {
+    IllustrationInfos r;
+    for ( const auto& ii : getIllustrationInfos() ) {
+      if ( ii.width == w && ii.height == h && ii.scale >= minScale ) {
+        r.push_back(ii);
+      }
+    }
+    return r;
+  }
+
   bool Archive::hasIllustration(unsigned int size) const {
     try {
       getIllustrationItem(size);
@@ -250,6 +318,15 @@ namespace zim
     throw EntryNotFound("Cannot find entry");
   }
 
+  Entry Archive::getEntryByPathWithNamespace(char ns, const std::string& path) const
+  {
+    auto r = m_impl->findx(ns, path);
+    if (r.first) {
+      return Entry(m_impl, entry_index_type(r.second));
+    }
+    throw EntryNotFound("Cannot find entry");
+  }
+
   Entry Archive::getEntryByTitle(entry_index_type idx) const
   {
     return Entry(m_impl, entry_index_type(m_impl->getIndexByTitle(title_index_t(idx))));
@@ -317,7 +394,7 @@ namespace zim
     auto entry = Entry(m_impl, entry_index_type(r.second));
     auto item = entry.getItem(true);
     auto accessInfo = item.getDirectAccessInformation();
-    return accessInfo.second;
+    return accessInfo.isValid();
   }
 
   bool Archive::hasTitleIndex() const {
@@ -328,7 +405,7 @@ namespace zim
     auto entry = Entry(m_impl, entry_index_type(r.second));
     auto item = entry.getItem(true);
     auto accessInfo = item.getDirectAccessInformation();
-    return accessInfo.second;
+    return accessInfo.isValid();
   }
 
   Archive::EntryRange<EntryOrder::pathOrder> Archive::iterByPath() const
@@ -487,6 +564,36 @@ namespace zim
     return m_impl->hasNewNamespaceScheme();
   }
 
+  size_t getClusterCacheMaxSize()
+  {
+    return getClusterCache().getMaxCost();
+  }
+
+  size_t getClusterCacheCurrentSize()
+  {
+    return getClusterCache().getCurrentCost();
+  }
+
+  void setClusterCacheMaxSize(size_t sizeInB)
+  {
+    getClusterCache().setMaxCost(sizeInB);
+  }
+
+  size_t Archive::getDirentCacheMaxSize() const
+  {
+    return m_impl->getDirentCacheMaxSize();
+  }
+
+  size_t Archive::getDirentCacheCurrentSize() const
+  {
+    return m_impl->getDirentCacheCurrentSize();
+  }
+
+  void Archive::setDirentCacheMaxSize(size_t nbDirents)
+  {
+    m_impl->setDirentCacheMaxSize(nbDirents);
+  }
+
   cluster_index_type Archive::getClusterCount() const
   {
     return cluster_index_type(m_impl->getCountClusters());
